
import it.sauronsoftware.ftp4j.FTPClient;
import it.sauronsoftware.ftp4j.FTPException;
import it.sauronsoftware.ftp4j.FTPIllegalReplyException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Utility class for building FTP {@link URL}s and {@link FTPClient}s. This
 * class follows the builder pattern where the builder is configured through a
 * series of setter calls and then instances of the target type are created with
 * the build methods. This class is thread-safe through simple synchronization.
 *
 * @author mpilone/edited by David Beecher
 */
public class FTPBuilder {

    private String host;
    private int port;
    private String username;
    private String password;

    /**
     * Sets the host of the remote FTP site.
     *
     * @param host the host name or IP address
     * @return the builder
     */
    public synchronized FTPBuilder setHost(String host) {
        this.host = host;
        return this;
    }

    /**
     * Sets the port of the remote FTP site.
     *
     * @param port the port number
     * @return the builder
     */
    public synchronized FTPBuilder setPort(int port) {
        this.port = port;
        return this;
    }

    /**
     * Sets the username for authentication on the remote FTP site.
     *
     * @param username the username for remote authentication
     * @return the builder
     */
    public synchronized FTPBuilder setUsername(String username) {
        this.username = username;
        return this;
    }

    /**
     * Sets the password for authentication on the remote FTP site.
     *
     * @param password the password for remote authentication
     * @return the builder
     */
    public synchronized FTPBuilder setPassword(String password) {
        this.password = password;
        return this;
    }

    /**
     * Builds a new {@link FTPClient}, connects to the remote host,
     * authenticates, and configures the connection for passive, binary
     * transfers.
     *
     * @return the new FTPClient instance
     * @throws IOException if any IO or FTP error occurs
     */
    public synchronized FTPClient buildClient() throws IOException {
        try {
            FTPClient client = new FTPClient();
            client.connect(host, port);
            client.login(username, password);
            client.setPassive(true);
            client.setType(FTPClient.TYPE_BINARY);

            return client;
        } catch (IllegalStateException | IOException | FTPIllegalReplyException | FTPException ex) {
            throw new IOException("Error building FTP client.", ex);
        }
    }

    /**
     * Builds a new {@link URL} using the configured host and port in the
     * builder. Note that the user information is not included in the URL.
     *
     * @param path the path to add to the end of the URL, starting with '/'
     * @return the new URL instance
     * @throws MalformedURLException if the URL cannot be built
     */
    public synchronized URL buildUrl(String path) throws MalformedURLException {
        return new URL(String.format("ftp://%s:%d%s", host, port, path));
    }

}
