
import it.sauronsoftware.ftp4j.FTPClient;
import it.sauronsoftware.ftp4j.FTPDataTransferListener;
import java.io.File;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Implementation of TransferJob to fetch a file from the FTP server
 */
public class GetJob extends TransferJob{

    private String m_RemoteFilename;
    private File m_LocalFile;
    private URL m_SrcURL = null;
    private URL m_DstURL = null;
    private FTPClient client;
    private int bytesTransferred = 0;
    private double fileSize;

    /**
     * Create a job to fetch a file from the FTP server
     *
     * @param ftpBuilder the FTP builder for creating remote connections
     * @param remoteFilename the name of the file on the FTP server to fetch
     */
    public GetJob(FTPBuilder ftpBuilder, String remoteFilename) {
        super(ftpBuilder);
        m_RemoteFilename = remoteFilename;
        try {
            m_LocalFile = new File(m_RemoteFilename); // Default to whatever current directory is

            // Pre-create URLs based on file names for this job
            m_SrcURL = ftpBuilder.buildUrl("/" + m_RemoteFilename);
            m_DstURL = m_LocalFile.toURI().toURL();
        } catch (Exception ex) {
            Logger.getLogger(GetJob.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * @return URL of remote file being fetched by this job
     */
    @Override
    public URL getSrcResource() {
        return m_SrcURL;
    }

    /**
     * @return URL of local file being stored by this job
     */
    @Override
    public URL getDestResource() {
        return m_DstURL;
    }

    /**
     * Synchronous implementation of the job transfer. Blocks until transfer is
     * completed, successfully or otherwise
     */
    @Override
    public void transfer() {
        try {
            client = ftpBuilder.buildClient();
            fileSize = client.fileSize(m_RemoteFilename);
            state = JobState.TRANSFERRING;  // This state will only be in effect "briefly"
            client.download(m_RemoteFilename, m_LocalFile, new FTPDataTransferListener() {

                @Override
                public void started() {
                    percentComplete = 0;
                }

                @Override
                public void transferred(int length) {
                    try {
                        bytesTransferred += length;
                        percentComplete = (double) bytesTransferred / fileSize;
                    } catch (IllegalStateException ex) {
                        Logger.getLogger(GetJob.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }

                @Override
                public void completed() {
                    percentComplete = 1;
                }

                @Override
                public void aborted() {
                    percentComplete = 0;
                }

                @Override
                public void failed() {
                    percentComplete = 0;
                }
            });
            state = JobState.SUCCESSFUL;
        } catch (Exception ex) {
            // Handle any exceptions by just setting job status to failed and caching exception message
            errorMessage = ex.getMessage();
            state = JobState.FAILED;
        }
    }

}
