
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * An implementation of TransferEngine that performs all operations in
 * separate threads. Therefore operations like addJob(…) will not block while
 * the job is added and executed.
 * 
 * David Beecher
 *
 */
public class MultiThreadedTransferEngine implements TransferEngine {

    /**
     * Jobs and Threads are kept in a map
     */
    private final Map<TransferJob, Thread> jobsAndThreads;    
 
    /**
     * Keep track of started status
     */
    private boolean started;

    /**
     * Default constructor
     */
    public MultiThreadedTransferEngine() {
        /**
         * Only need to create an empty list of jobs and threads here
         */
        jobsAndThreads = new LinkedHashMap<>();
    }

    /**
     * @return Internal list of transfer jobs
     */
    @Override
    public Collection<TransferJob> getJobs() {
        return jobsAndThreads.keySet();
    }

    /**
     * Adds a job to the engine. The job starts immediately. Jobs added to the
     * engine must be in the {@link JobState#READY} state.
     *
     * @param job The TransferJob object to process
     */
    @Override
    public void addJob(TransferJob job) {
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                job.transfer();
            }
        });
        thread.start();
        jobsAndThreads.put(job, thread);

    }

    @Override
    public void start() {
        if (!started) {
            /**
             * If there are any READY jobs on the list, transfer them right away
             * in a separate thread.
             */
            for (TransferJob job : jobsAndThreads.keySet()) {
                if (job.getState() == JobState.READY) {
                    jobsAndThreads.get(job).start();
                }
            }
            started = true;
        }
    }

    /**
     * Stops the engine. Wait for all of the threads to finish. Then sets internal state to "not started"
     */
    @Override
    public void stop() {
        // Wait for all of the threads to finish.
        for (Thread thread : jobsAndThreads.values()) {
            try {
                thread.join();
            } catch (InterruptedException ex) {
            }
        }
        started = false;
    }

}
