
import it.sauronsoftware.ftp4j.FTPClient;
import it.sauronsoftware.ftp4j.FTPDataTransferListener;
import java.io.File;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Implementation of a TransferJob to upload a file to the FTP server
 */
public class PutJob extends TransferJob {

    private String m_Filename;
    private File m_LocalFile;
    private URL m_SrcURL = null;
    private URL m_DstURL = null;
    private long fileSize;
    private int bytesTransferred = 0;

    /**
     * Create a job to upload a file to the FTP server
     *
     * @param ftpBuilder the FTP builder for creating remote connections
     * @param localFilename the name of the local file to upload to the FTP
     * server
     */
    public PutJob(FTPBuilder ftpBuilder, String localFilename) {
        super(ftpBuilder);
        try {
            m_Filename = localFilename;
            m_LocalFile = new File(m_Filename);
            // Pre-create URLs based on file names for this job
            m_SrcURL = m_LocalFile.toURI().toURL();
            m_DstURL = ftpBuilder.buildUrl("/" + m_Filename);
        } catch (Exception ex) {
            Logger.getLogger(PutJob.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * @return URL of local file being uploaded by this job
     */
    @Override
    public URL getSrcResource() {
        return m_SrcURL;
    }

    /**
     * @return URL of file on FTP server being uploaded by this job
     */
    @Override
    public URL getDestResource() {
        return m_DstURL;
    }

    /**
     * Synchronous implementation of the job transfer. Blocks until transfer is
     * completed, successfully or otherwise
     */
    @Override
    public void transfer() {
        try {
            FTPClient client = ftpBuilder.buildClient();
            fileSize = client.fileSize(m_Filename);
            state = JobState.TRANSFERRING;
            client.upload(m_LocalFile, new FTPDataTransferListener() {

                @Override
                public void started() {
                    percentComplete = 0;
                }

                @Override
                public void transferred(int length) {
                    try {
                        bytesTransferred += length;
                        percentComplete = (double) bytesTransferred / fileSize;
                    } catch (IllegalStateException ex) {
                        Logger.getLogger(GetJob.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }

                @Override
                public void completed() {
                    percentComplete = 1;
                }

                @Override
                public void aborted() {
                    percentComplete = 0;
                }

                @Override
                public void failed() {
                    percentComplete = 0;
                }
            });
            state = JobState.SUCCESSFUL;
        } catch (Exception ex) {
            // Handle any exceptions by just setting job status to failed and caching exception message
            errorMessage = ex.getMessage();
            state = JobState.FAILED;
        }
    }
}
