
import java.util.Collection;
import java.util.LinkedList;

/**
 * An implementation of TransferEngine that performs all operations in a single
 * thread (i.e  the same thread as the caller). Therefore operations like
 * addJob(…) will block while the job is added and executed.
 *
 */
public class SingleThreadedTransferEngine
        implements TransferEngine {

    /**
     * Jobs are kept in a simple linked list
     */
    private LinkedList<TransferJob> m_Jobs;

    /**
     * Keep track of started status
     */
    private boolean m_Started = false;

    /**
     * Default constructor
     */
    public SingleThreadedTransferEngine() {
        /**
         * Only need to create an empty list of jobs here
         */
        m_Jobs = new LinkedList<TransferJob>();
    }

    /**
     * @return Internal list of transfer jobs
     */
    @Override
    public Collection<TransferJob> getJobs() {
        return m_Jobs;
    }

    /**
     * Adds a job to the engine. The job starts immediately. Jobs added to the
     * engine must be in the {@link JobState#READY} state.
     *
     * @param job The TransferJob object to process
     */
    @Override
    public void addJob(TransferJob job) {
        m_Jobs.add(job);
        job.transfer();
    }

    /**
     *
     */
    @Override
    public void start() {
        if (m_Started) {
            return;
        }

        /**
         * If there are any READY jobs on the list, transfer them right away.
         */
        for (TransferJob job : m_Jobs) {
            if (job.getState() == JobState.READY) {
                job.transfer();
            }
        }
        m_Started = true;
    }

    /**
     * Stops the engine. Since all jobs are synchronous, there will never be
     * anything to wait for. Simply sets internal state to "not started"
     */
    @Override
    public void stop() {
        if (!m_Started) {
            return;
        }
        m_Started = false;
    }

}
