
import java.net.MalformedURLException;
import java.net.URL;

/**
 * The base class for all jobs that perform a transfer to or from a remote site.
 *
 * @author mpilone
 */
public abstract class TransferJob {

    /**
     * The FTP builder for creating remote connections
     */
    protected final FTPBuilder ftpBuilder;

    /**
     * The state of the job. The default initial state is
     * {@link JobState#READY}.
     */
    protected volatile JobState state;

    /**
     * The amount of the transfer that is complete from 0 to 1.
     */
    protected volatile double percentComplete;

    /**
     * The error message if the job state is FAILED.
     */
    protected volatile String errorMessage;

    /**
     * Constructs the transfer job which will transfer the source resource to
     * the destination resource.
     *
     * @param ftpBuilder the FTP builder for creating remote connections
     */
    public TransferJob(FTPBuilder ftpBuilder) {
        this.ftpBuilder = ftpBuilder;
        this.state = JobState.READY;
    }

    /**
     * Transfers the source file to the destination file. This method blocks
     * until the transfer is complete, fails, or is aborted.
     */
    public abstract void transfer();

    /**
     * Returns the state of the job.
     *
     * @return the state of the job
     */
    public JobState getState() {
        return state;
    }

    /**
     * Returns the percent complete as a value between 0 and 1.
     *
     * @return the percent complete
     */
    public double getPercentComplete() {
        return percentComplete;
    }

    /**
     * Returns the error message if the job is in state {@link JobState#FAILED}.
     *
     * @return the error message or null
     */
    public String getErrorMessage() {
        return errorMessage;
    }

    /**
     * Returns the destination resource URL. For example, if the transfer is
     * downloading a file, the destination resource will be a local file URL.
     *
     * @return the destination resource
     * @throws java.net.MalformedURLException if the URL cannot be constructed
     */
    public abstract URL getDestResource() throws MalformedURLException;

    /**
     * Returns the source resource URL. For example, if the transfer is
     * downloading a file, the source resource will be a remote file URL.
     *
     * @return the source resource
     * @throws java.net.MalformedURLException if the URL cannot be constructed
     */
    public abstract URL getSrcResource() throws MalformedURLException;
}
